/***************************************************************************
                          qdvbzapchannellist.cpp  -  description
                             -------------------
    begin                : Mon Jun 30 2003
    copyright            : (C) 2003 by Andre Stein
    email                : stone@steinsoft.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "qdvbzapchannellist.h"
#include "qdvbzap.h"
#include <qapplication.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <iostream>

using namespace std;

//wakeupdelay that is globally set for all tooltips
//(less is faster)
#define TOOLTIP_WAKEUPDELAY 1

const QString CONFIG_DIRECTORY = QString(getenv("HOME")) + "/.qdvbzap/";
const QString CONFIG_FILE_PATH  = CONFIG_DIRECTORY + "channels.conf";

/***************************************************************************
   constructor/destructor
 ***************************************************************************/

QdvbzapChannelList::QdvbzapChannelList(QWidget *parent, const char *name )
: QListBox(parent,name)
{
	/**
	 load saved channels
	 **/
	system("mkdir " + CONFIG_DIRECTORY);   // create qdvbzap config directory (if not yet)
	channelList.loadFrom(CONFIG_FILE_PATH);

        /**************/
	
	//recreate list at first
	reCreateList();
	//setup tooltip that will show information about
	//the currently _moused_ channel
	QdvbzapChannelTooltip* tooltip = new QdvbzapChannelTooltip(this);
	tooltip->setWakeUpDelay(TOOLTIP_WAKEUPDELAY);

	//setup slots
	QObject::connect(this, SIGNAL(doubleClicked(QListBoxItem*)),
			 this, SLOT(slotDoubleClicked(QListBoxItem*)));

	/* create popupmenu */
        popupMenu = new QPopupMenu(this);
        popupMenu->insertItem("&Watch now!", this, SLOT(slotWatchCurrentChannel()));
        popupMenu->insertItem("&Delete channel", this, SLOT(slotDeleteCurrent()));
        popupMenu->insertSeparator();
        popupMenu->insertItem("&Clear channels", this, SLOT(slotClearList()));
        popupMenu->insertItem("&Import channels...", this, SLOT(slotImportChannels()));

}

QdvbzapChannelList::~QdvbzapChannelList()
{
}

/***************************************************************************
   functions
 ***************************************************************************/
 
/** recreates list of channels */
void QdvbzapChannelList::reCreateList()
{
	SChannel* channel;
	//clear list
	clear();

	//go through channels and update listbox
	while ( (channel = channelList.iterateNextChannel()) )
	{
		insertItem(channel->name);
	} 
}

/** slot for signak doubleClicked() */
void QdvbzapChannelList::slotDoubleClicked(QListBoxItem* item)
{
	//TODO: depending on used frontend application
	///////////////////

	//use SChannelView to start watching the channel that has been
	//double-clicked
	//  only if real item has been chosen
        if (item)
        {
		//watch channel that the channelist finds for
		//the item we have selected.. really 3lit3
		channelViewer.watchChannel(channelList.lookup(item->text()));

		//little message..
		((QMainWindow*)qApp->mainWidget())->statusBar()->message(
			QString("Watching %1 ..").arg(item->text()), 2000);
	}
}

/** view currently selected channel using currently selected frontend app */
void QdvbzapChannelList::slotWatchCurrentChannel()
{
	//get current item's text and watch it. yo
	SChannel* channel = channelList.lookup(currentText());

	if (channel)
	{
		channelViewer.watchChannel(channel);
		//little message..
		((QMainWindow*)qApp->mainWidget())->statusBar()->message(
			QString("Watching %1 ..").arg(channel->name), 2000);
	}
}

/** overwritten function to display a popup menu */
void QdvbzapChannelList::contextMenuEvent ( QContextMenuEvent * e )
{
	//let the menu popup
	popupMenu->popup(e->globalPos());

	//parent window will do the rest
	QListBox::contextMenuEvent(e);
}

/** slot functions that imports channels from an exisiting file */
void QdvbzapChannelList::slotImportChannels()
{
	//bring a file dialog up and wait what happens
	QString myFile = QFileDialog::getOpenFileName(
			getenv("HOME"),
			"All Files(*)",
			this);
	//if not null string
	if (!myFile.isEmpty())
	{
		//load da channels and add ta list
		channelList.loadFrom(myFile);
                //update list..
		reCreateList();
	}
}

/** No descriptions */
void QdvbzapChannelList::slotClearList()
{
	if (QMessageBox::information(this, "Clear channels",
	 "<b>Are you really sure?</b><br><br>This will delete all channels in your list.",
	 QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes)
	{
		//if you don't know what's happening here, give up programming..
		clear();
		channelList.clear();
	}
}

/** No descriptions */
void QdvbzapChannelList::slotDeleteCurrent()
{
	//bye bye currently selected item
	channelList.removeChannel(currentText());
	//update list
	reCreateList();
}

/** saves everything
 */
void QdvbzapChannelList::save()
{
	channelList.saveTo(CONFIG_FILE_PATH);
}

/** stops currently active frontend app */
void QdvbzapChannelList::slotStopWatching()
{
	channelViewer.stopWatching();
}
