/***************************************************************************
                          schannelviewer.cpp  -  description
                             -------------------
    begin                : Mon Jun 30 2003
    copyright            : (C) 2003 by Andre Stein
    email                : stone@steinsoft.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "schannelviewer.h"
#include <iostream>
#include <qapp.h>
#include <fstream>
#include <stdlib.h>

using namespace std;

const char* TEMPFILE_NAME = "/tmp/qdvbzap_command.sh";

/***************************************************************************
   constructor/destructor
 ***************************************************************************/

SChannelViewer::SChannelViewer()
: activeApp(0), isRunning(false)
{
       	addApplication("Xine", "xine stdin://", "xine");
       	addApplication("MPlayer", "mplayer -cache 4096", "mplayer");
	addApplication("Save to ~/qdvbzap.mpg", "cat > ~/qdvbzap.mpg", "dvbstream");
}

SChannelViewer::~SChannelViewer()
{
}

/***************************************************************************
   functions
 ***************************************************************************/

/** start watching selected channel */
void SChannelViewer::watchChannel(SChannel* channel)
{
	//allow only one running instance, so isRunning must be false
	//if not stop currently running
	if (isRunning)
	{
		stopWatching();
	}

	/************************
	  STEPS:
	    1. create a temporary file that holds our command
	        (depending on channel and chosen frontend apps)
	    2. run this temorary script using system(..) and sh
	 ************************/

	//create file
	ofstream file(TEMPFILE_NAME, ios::out | ios::trunc);

	//STEP 1
	if (file)
	{
		//standard bash things..
		file << "#!/bin/sh" << endl << endl;
		
		//construct command sequence and write to file
		file << "dvbstream -f " << channel->frequency
		     << " -p " << (channel->polarisation == POL_H ? 'h' : 'v')
		     << " -s " << channel->symbolRate
		     << " -o " << channel->pids[0] << " "
		     << channel->pids[1] << " " << channel->pids[2]
		     << " " << channel->pids[3] << " | ";

		//after dvstream, the chosen frontendapp
		if (apps.size() != 0)
			file << apps[activeApp].command << endl;

		//written. done.
	}
	else
	{
		cerr << "Qdvbzap: error  couldn't open template for writing." << endl;
		return;
	}

	//STEP 2
	// run command using system(..)

	//enjoy watching..    
	if(system(QString("sh ") + TEMPFILE_NAME + " &") == -1)
	{
		cerr << "Qdvbzap: error  for some reason couldn't start watching." << endl;
		return;
	}

	//itze running
	isRunning = true;
}

/** adds a new frontend application */
void SChannelViewer::addApplication(QString name, QString command, QString processName)
{
	//setup strcuture
	SFrontendApp toAdd;
	toAdd.name = name;
	toAdd.command = command;
	toAdd.processName = processName;

	//add it
	apps.push_back(toAdd);
}

/** sets active frontend application */
void SChannelViewer::setActiveApplication(const QString& appName)
{
	for (int i = 0; i < apps.size(); ++i)
	{
		if (apps[i].name == appName)
		{
			activeApp = i;
			return;
		}
	}
}

/** gets next application in list, 0 for end */
SFrontendApp* SChannelViewer::iterateNextApplication()
{
	static int i = -1;

        //next one
        ++i;

	//if we have iterated through array,
	//reset and return 0 for done.
	if ( i >= apps.size() )
	{
		i = 0;
		return 0;
	}

	//return current
	return &apps[i];
}

/** terminate currently running frontend app */
void SChannelViewer::stopWatching()
{
	if (!isRunning)
		return;

	//terminate process and reset everything
	system(QString("killall ") + apps[activeApp].processName);
	isRunning = false;
}